//Copyright (c) 1997 The Open Group
//
//Permission to use, copy, modify, and distribute this software and its documentation for
//any purpose and without fee is hereby granted, provided that the above copyright notice
//appears in all copies and that both the copyright notice and this permission notice
//appear in supporting documentation, and that the name of ("TOG") or The Open Group not
//be used in advertising or publicity pertaining to distribution of the software without
//specific, written prior permission.
//
//THE OPEN GROUP DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT
//SHALL TOG BE LIABLE FOR ANY SPECIAL, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
//WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT,
//NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//PERFORMANCE OF THIS SOFTWARE. 


package ORG.opengroup.misc;

import java.util.*;

/**
 * A structure that maintains two hashtables.
 * One is the inverse of the other
 */
public class DoubleHashtable
{
  Hashtable ht = null;
  Hashtable inverseHt = null;

	/**
	 * Creates a new DoublehashTable
	 */
  public DoubleHashtable(){
    ht = new Hashtable();
    inverseHt = new Hashtable();
  }

	/**
	 * Adds a new (key, value) pair to the structure
	 */
  public synchronized void put(Object key, Object value){
    ht.put(key,value);
    inverseHt.put(value,key);
  }

	/**
	 * Retrieves a (key, value) pair from the structure
	 */
  public synchronized Object get(Object key){
    return ht.get(key);
  }

	/**
	 * Retrieves the inverse of a (key, value) pair from the structure
	 */
  public synchronized Object getInverse(Object key){
    return inverseHt.get(key);
  }

	/**
	 * Removes a (key,value) pair
	 */
  public synchronized Object remove(Object key){
    Object value = ht.remove(key);
    if (value != null) inverseHt.remove(value);
    return value;
  }

	/**
	 * Removes an inverse (key,value) pair
	 */
  public synchronized Object removeInv(Object key){
    Object value = inverseHt.remove(key);
    if (value != null) ht.remove(value);
    return value;
  }

} // DoubleHashtable

  

  
